import { InterfaceError } from "./interface-error.class";
import { Subject } from "rxjs";
/**
 * Base external interface
 */
var ExternalInterface = /** @class */ (function () {
    function ExternalInterface() {
        this.unexpectedLogoutSubject = new Subject();
        /**
         * if true, the save method will only send the modified properties to the service
         * @type {boolean}
         */
        this.useDiff = false;
    }
    /**
     * Load an entity from the service
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {EntityDataSet | Observable<EntityDataSet>} A set of data, or an observable
     */
    ExternalInterface.prototype.loadEntity = function (type, id, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        console.warn("LoadEntity not implemented in interface");
        return null;
    };
    /**
     * Load an entity collection from the service
     * @param {string} type Name of the endpoint
     * @param {FilterData} filter Collection filter object
     * @param {Function} errorHandler Function used to handle errors
     * @returns {CollectionDataSet | Observable<CollectionDataSet>} A collection set of data, or an observable
     */
    ExternalInterface.prototype.loadCollection = function (type, filter, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        console.warn("LoadCollection not implemented in interface");
        return null;
    };
    /**
     *
     */
    ExternalInterface.prototype.clear = function () {
        console.warn("LoadCollection not implemented in interface");
    };
    /**
     *
     * @param {string} type
     * @param {CollectionOptionsInterface} options
     * @returns {CollectionDataSet | Observable<CollectionDataSet>}
     */
    ExternalInterface.prototype.paginatedLoadCollection = function (type, options, paginator, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        console.warn("PaginatedLoadCollection not implemented in interface");
        return null;
    };
    /**
     * Create an entity on the service
     * @param {string} type Endpoint name
     * @param {EntityDataSet} data Base data used to create the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {EntityDataSet | Observable<EntityDataSet>} A set of data, or an observable
     */
    ExternalInterface.prototype.createEntity = function (type, data, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        console.warn("CreateEntity not implemented in interface");
        return null;
    };
    /**
     * Delete an entity from the service
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {boolean | Observable<boolean>} True if deletion success
     */
    ExternalInterface.prototype.deleteEntity = function (type, id, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        console.warn("DeleteEntity not implemented in interface");
        return null;
    };
    /**
     * Save an entity on the service
     * @param {EntityDataSet} data Data to Save
     * @param {string} type Name of the endpoint
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {EntityDataSet | Observable<EntityDataSet>} The saved data
     */
    ExternalInterface.prototype.saveEntity = function (data, type, id, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        console.warn("SaveEntity not implemented in interface");
        return null;
    };
    /**
     * Authenticating to the service
     * @param {string} login User login
     * @param {string} password User password
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<boolean>}
     */
    ExternalInterface.prototype.authenticate = function (login, password, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        console.warn("Authenticate not implemented in interface");
        return null;
    };
    ExternalInterface.prototype.logout = function () {
        return null;
    };
    /**
     * Release an endpoint if not useful anymore
     * @param {string} type Name of the endpoint
     */
    ExternalInterface.prototype.release = function (type) {
        console.warn("Release not implemented in interface");
    };
    /**
     * Sends an error message
     * @param {number} code Error code
     * @param {string} originalMessage Error original text message
     * @param {Function} errorHandler Error handler Function
     */
    ExternalInterface.prototype.sendError = function (code, originalMessage, errorHandler, data) {
        if (data === void 0) { data = {}; }
        var error = new InterfaceError(code, "", originalMessage, data);
        errorHandler(error);
    };
    return ExternalInterface;
}());
export { ExternalInterface };
//# sourceMappingURL=abstract-external-interface.class.js.map