var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ExternalInterface } from "../abstract-external-interface.class";
import { Observable, ReplaySubject } from "rxjs/Rx";
/**
 * Http external interface
 */
var Drupal8 = /** @class */ (function (_super) {
    __extends(Drupal8, _super);
    /**
     * Creates the http interface
     * @param {HttpConfiguration} configuration Configuration object
     * @param {DataConnector} connector Reference to the connector
     */
    function Drupal8(configuration, connector, interfaceName) {
        var _this = _super.call(this) || this;
        _this.configuration = configuration;
        _this.connector = connector;
        _this.interfaceName = interfaceName;
        /*
        Headers sent with each request
         */
        _this.headers = {};
        _this.useDiff = true;
        if (configuration.headers) {
            for (var header in configuration.headers) {
                if (configuration.headers.hasOwnProperty(header)) {
                    _this.headers[header] = configuration.headers[header];
                }
            }
        }
        _this.dataStore = {
            user: undefined
        };
        return _this;
    }
    Object.defineProperty(Drupal8.prototype, "authenticated", {
        /**
         * Is the user authenticated on this service ?
         * @returns {Observable<EntityDataSet>}
         */
        get: function () {
            var value = new ReplaySubject(1);
            this.dataStore.user = JSON.parse(localStorage.getItem(this.interfaceName + "_currentUser"));
            var expire = JSON.parse(localStorage.getItem(this.interfaceName + "_expires_in"));
            if (expire > Date.now()) {
                this.dataStore.user = JSON.parse(localStorage.getItem(this.interfaceName + "_currentUser"));
                this.setToken(JSON.parse(localStorage.getItem(this.interfaceName + "_accessToken"))).subscribe(function (data) {
                    value.next(data);
                });
            }
            else if (expire && expire < Date.now()) {
                value.error(null);
                this.logOut();
            }
            else {
                value.error(null);
            }
            return value;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Add headers to the request
     * @param {XMLHttpRequest} request A xhr request
     */
    Drupal8.prototype.addHeaders = function (request) {
        for (var headerName in this.headers) {
            request.setRequestHeader(headerName, this.headers[headerName]);
        }
    };
    /**
     * Load entity in http service
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the data
     */
    Drupal8.prototype.loadEntity = function (type, id, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.configuration.apiUrl + type + "/" + id;
        request.open("GET", url, true);
        var subject = new ReplaySubject(1);
        this.addHeaders(request);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    subject.next(_this.extractEntity(request.responseText));
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler);
                }
            }
        };
        request.send();
        return subject;
    };
    /**
     * Load a collection in http service
     * @param {string} type Endpoint name
     * @param {{[p: string]: any}} filter Filter Object
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<CollectionDataSet>} Observable returning the collection data
     */
    Drupal8.prototype.loadCollection = function (type, filter, errorHandler) {
        var _this = this;
        if (filter === void 0) { filter = {}; }
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.configuration.apiUrl + type;
        var filterKeys = Object.keys(filter);
        if (filterKeys.length > 0) {
            url += "?";
        }
        filterKeys.forEach(function (key, index) {
            var val = filter[key];
            url += "filter[" + key + "]=" + val;
            if (index < filterKeys.length - 1) {
                url += "&";
            }
        });
        request.open("GET", url, true);
        this.addHeaders(request);
        var subject = new ReplaySubject(1);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    subject.next(_this.extractCollection(request.responseText));
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler);
                }
            }
        };
        request.send();
        return subject;
    };
    /**
     * Save entity to the http service
     * @param {EntityDataSet} entity Entity data to save
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the entity data
     */
    Drupal8.prototype.saveEntity = function (entity, type, id, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.configuration.apiUrl + type + "/" + id;
        request.open("PATCH", url, true);
        this.addHeaders(request);
        var subject = new ReplaySubject(1);
        var saveObject = {
            data: {
                id: id,
                type: type,
                attributes: entity
            }
        };
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    subject.next(_this.extractEntity(request.responseText));
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler);
                }
            }
        };
        request.send(JSON.stringify(saveObject));
        return subject;
    };
    /**
     * Create entity in http service
     * @param {string} type Endpoint name
     * @param {EntityDataSet} data Data used to create the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the entity data
     */
    Drupal8.prototype.createEntity = function (type, data, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.configuration.apiUrl + type;
        request.open("POST", url, true);
        this.addHeaders(request);
        var addObject = {
            data: {
                attributes: data,
                type: type
            }
        };
        var subject = new ReplaySubject(1);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 201) {
                    subject.next(_this.extractEntity(request.responseText));
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler);
                }
            }
        };
        request.send(JSON.stringify(addObject));
        return subject;
    };
    /**
     * Delete entity from http service
     * @param {string} type Endpoint type
     * @param {number} id Entity id
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<boolean>} True if deletion success
     */
    Drupal8.prototype.deleteEntity = function (type, id, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.configuration.apiUrl + type + "/" + id;
        request.open("DELETE", url, true);
        this.addHeaders(request);
        var subject = new ReplaySubject(1);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 204) {
                    subject.next(true);
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler);
                }
            }
        };
        request.send();
        return subject;
    };
    Drupal8.prototype.getOauthToken = function (login, password) {
        var subject = new ReplaySubject(1);
        var req = new XMLHttpRequest();
        var url = this.configuration.apiUrl + "oauth/token";
        req.open("POST", url, true);
        req.onreadystatechange = function () {
            if (req.readyState === XMLHttpRequest.DONE) {
                if (req.status === 200) {
                    subject.next(JSON.parse(req.responseText));
                }
                else {
                    subject.error(req.responseText);
                }
            }
        };
        var data = {
            grant_type: "password",
            client_id: this.configuration.clientId,
            userName: login,
            password: password,
            scope: this.configuration.scope || "administrator angular"
        };
        if (this.configuration.clientSecret) {
            data["client_secret"] = this.configuration.clientSecret;
        }
        req.send(data);
        return subject;
    };
    /**
     * Authenticate in service
     * @param {string} login User login
     * @param {string} password User password
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<boolean>} True if authentication success
     */
    Drupal8.prototype.authenticate = function (login, password, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var subject = new ReplaySubject(1);
        var request = new XMLHttpRequest();
        var url = this.configuration.apiUrl + "oauth/token";
        request.open("POST", url, true);
        request.setRequestHeader("content-type", "application/x-www-form-urlencoded");
        //request.setRequestHeader("Authorization", 'Basic ' + btoa(login.trim() + ':' + password));
        var observables = [];
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    localStorage[_this.interfaceName + "_userName"] = login;
                    var loginData = JSON.parse(request.responseText);
                    var expire = +loginData["expires_in"] - 3600;
                    if (expire < 3600) {
                        if (localStorage.getItem(_this.interfaceName + "_accessToken")) {
                            observables.push(_this.setToken(loginData["access_token"], errorHandler));
                            _this.setExpireDate(expire);
                            _this.setRefreshToken(loginData["refresh_token"]);
                        }
                        observables.push(_this.refreshToken(loginData["refresh_token"], errorHandler));
                    }
                    else {
                        observables.push(_this.setToken(loginData["access_token"], errorHandler));
                        _this.setExpireDate(expire);
                        _this.setRefreshToken(loginData["refresh_token"]);
                    }
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler);
                }
                Observable.combineLatest.apply(Observable, observables).map(function (values) {
                    return values[0];
                }).subscribe(function (data) {
                    subject.next(data);
                });
            }
        };
        var data = {
            grant_type: "password",
            client_id: this.configuration.clientId,
            username: login,
            password: password,
            scope: this.configuration.scope || "administrator angular"
        };
        if (this.configuration.clientSecret) {
            data["client_secret"] = this.configuration.clientSecret;
        }
        var dataStr = "";
        for (var id in data) {
            dataStr += id + "=" + data[id] + "&";
        }
        request.send(dataStr);
        return subject;
    };
    Drupal8.prototype.logOut = function () {
        // TODO: revoir cette partie du logout
        /*let keys:string[] = Object.keys(this.headers);

        keys.forEach((headerName:string) => {
            if (headerName !== "access-token") {
                delete this.headers[headerName];
            }
        });*/
        localStorage.removeItem(this.interfaceName + "_currentUser");
        localStorage.removeItem(this.interfaceName + "_accessToken");
        localStorage.removeItem(this.interfaceName + "_expires_in");
        localStorage.removeItem(this.interfaceName + "_refreshToken");
        this.dataStore.user = null;
    };
    /**
     *
     * @param {string} accessToken
     * @param {Function} errorHandler
     */
    Drupal8.prototype.setToken = function (accessToken, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        if (accessToken && accessToken != "") {
            localStorage.setItem(this.interfaceName + "_accessToken", JSON.stringify(accessToken));
            this.headers["Authorization"] = "Bearer " + accessToken;
            return this.getMe(true, errorHandler);
        }
    };
    /**
     *
     * @param {number} expire
     */
    Drupal8.prototype.setExpireDate = function (expire) {
        var date = Date.now();
        localStorage.setItem(this.interfaceName + "_expires_in", JSON.stringify(date + (expire * 1000)));
    };
    /**
     *
     * @param {string} refreshToken
     * @param {Function} errorHandler
     */
    Drupal8.prototype.refreshToken = function (refreshToken, errorHandler) {
        var _this = this;
        var subject = new ReplaySubject(1);
        var request = new XMLHttpRequest();
        var url = this.configuration.apiUrl + "refresh-token/" + refreshToken;
        request.open("GET", url, true);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    var userData = JSON.parse(request.responseText);
                    _this.setToken(userData["access_token"], errorHandler);
                    _this.setExpireDate(+userData["expires_in"] - 3600);
                    _this.setRefreshToken(userData["refresh_token"]);
                    subject.next(userData);
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler);
                }
            }
        };
        request.send();
        return subject;
    };
    /**
     *
     * @param {string} refreshToken
     */
    Drupal8.prototype.setRefreshToken = function (refreshToken) {
        localStorage.setItem(this.interfaceName + "_refreshToken", JSON.stringify(refreshToken));
    };
    /**
     *
     * @param {boolean} complete
     * @param {Function} errorHandler
     * @returns {Observable<EntityDataSet>}
     */
    Drupal8.prototype.getMe = function (complete, errorHandler) {
        var _this = this;
        if (complete === void 0) { complete = true; }
        if (errorHandler === void 0) { errorHandler = null; }
        var subject = new ReplaySubject(1);
        var request = new XMLHttpRequest();
        var url = this.configuration.apiUrl + "user?filter[name][value]=" + localStorage[this.interfaceName + "_userName"];
        request.open("GET", url, true);
        this.addHeaders(request);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    var userData = JSON.parse(request.responseText)["data"][0]["attributes"];
                    userData["id"] = userData["uuid"];
                    subject.next(userData);
                    _this.setMe(userData, complete);
                }
                else {
                    if (errorHandler) {
                        _this.sendError(request.status, request.statusText, errorHandler);
                    }
                    subject.error(null);
                }
            }
        };
        request.send();
        return subject;
    };
    /**
     *
     * @param {EntityDataSet} userData
     * @param {boolean} complete
     */
    Drupal8.prototype.setMe = function (userData, complete) {
        if (complete === void 0) { complete = true; }
        if (complete) {
            this.dataStore.user = userData;
            //this.data.next(this.dataStore.user);
            localStorage.setItem(this.interfaceName + "_currentUser", JSON.stringify(userData));
        }
        //this.currentUserData = userData;
    };
    /**
     * Extract entity data from raw data
     * @param {string} responseText Response text from server
     * @returns {EntityDataSet} Entity data
     */
    Drupal8.prototype.extractEntity = function (responseText) {
        var data = JSON.parse(responseText)["data"]["attributes"];
        data["id"] = data["uuid"];
        return data;
    };
    /**
     * Extract collection data from raw data
     * @param {string} responseText Response text from server
     * @returns {CollectionDataSet} Collection data
     */
    Drupal8.prototype.extractCollection = function (responseText) {
        var data = JSON.parse(responseText);
        var collectionData = {};
        data["data"].forEach(function (entityData) {
            entityData["attributes"]["id"] = entityData["attributes"]["uuid"];
            collectionData[entityData["attributes"]["uuid"]] = entityData["attributes"];
        });
        return collectionData;
    };
    return Drupal8;
}(ExternalInterface));
export { Drupal8 };
//# sourceMappingURL=drupal8.class.js.map