var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ExternalInterface } from "../abstract-external-interface.class";
import { BehaviorSubject, ReplaySubject } from "rxjs/Rx";
import { combineLatest } from 'rxjs/observable/combineLatest';
import { OrderDirection } from "../../order-direction.enum";
/**
 * Http external interface
 */
var Http = /** @class */ (function (_super) {
    __extends(Http, _super);
    /**
     * Creates the http interface
     * @param {HttpConfiguration} configuration Configuration object
     * @param {DataConnector} connector Reference to the connector
     */
    function Http(configuration, connector, interfaceName) {
        var _this = _super.call(this) || this;
        _this.configuration = configuration;
        _this.connector = connector;
        _this.interfaceName = interfaceName;
        /*
        Headers sent with each request
         */
        _this.headers = {};
        _this.useDiff = true;
        _this.clear();
        window.addEventListener("storage", function (event) { return _this.OnUnexpectedStorageChange(event); });
        return _this;
    }
    /**
     *
     */
    Http.prototype.clear = function () {
        if (this.configuration.headers) {
            for (var header in this.configuration.headers) {
                if (this.configuration.headers.hasOwnProperty(header)) {
                    this.headers[header] = this.configuration.headers[header];
                }
            }
        }
        this.dataStore = {
            user: undefined
        };
    };
    Object.defineProperty(Http.prototype, "authenticated", {
        /**
         * Is the user authenticated on this service ?
         * @returns {Observable<EntityDataSet>}
         */
        get: function () {
            var value = new ReplaySubject(1);
            this.dataStore.user = JSON.parse(localStorage.getItem(this.interfaceName + "_currentUser"));
            var expire = JSON.parse(localStorage.getItem(this.interfaceName + "_expires_in"));
            if (expire > Date.now()) {
                this.dataStore.user = JSON.parse(localStorage.getItem(this.interfaceName + "_currentUser"));
                this.setToken(JSON.parse(localStorage.getItem(this.interfaceName + "_accessToken"))).subscribe(function (data) {
                    value.next(data);
                });
            }
            else if (expire && expire < Date.now()) {
                value.error(null);
                this.logout();
            }
            else {
                value.error(null);
            }
            return value;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Add headers to the request
     * @param {XMLHttpRequest} request A xhr request
     */
    Http.prototype.addHeaders = function (request) {
        for (var headerName in this.headers) {
            if (this.headers.hasOwnProperty(headerName)) {
                request.setRequestHeader(headerName, this.headers[headerName]);
            }
        }
    };
    Http.prototype.apiUrl = function (endpointName) {
        var useApi = !this.configuration.useApiExtension === false;
        var ext = useApi ? "api/" : "";
        var endPointConf = this.connector.getEndpointConfiguration(endpointName);
        var useLanguage = false;
        if (endPointConf && typeof endPointConf === "object") {
            useLanguage = endPointConf.useLanguage;
        }
        if (typeof this.configuration.apiUrl === "string") {
            if (!useLanguage) {
                return this.configuration.apiUrl + ext;
            }
            else {
                return this.configuration.apiUrl + this.connector.currentLanguage + "/" + ext;
            }
        }
    };
    /**
     * Load entity in http service
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the data
     */
    Http.prototype.loadEntity = function (type, id, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.apiUrl(type) + type + "/" + id;
        request.open("GET", url, true);
        var subject = new ReplaySubject(1);
        this.addHeaders(request);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    subject.next(_this.extractEntity(request.responseText));
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler, {
                        entityType: type,
                        entityId: id,
                        response: JSON.parse(request.responseText)
                    });
                }
            }
        };
        request.send();
        return subject;
    };
    Http.prototype.paginatedLoadCollection = function (type, options, paginator, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.apiUrl(type) + type;
        if (options.urlExtension) {
            if (options.urlExtension.charAt(0) !== "/") {
                url += "/";
            }
            url += options.urlExtension;
        }
        var orderOptionsLength = options.orderOptions ? options.orderOptions.length : 0;
        var filtersLength = options.filter ? Object.keys(options.filter).length : 0;
        if (filtersLength > 0 || orderOptionsLength > 0 || options.offset || options.range || options.page) {
            url += '?';
        }
        var started = false;
        if (orderOptionsLength > 0) {
            started = true;
            url += 'sort=';
            options.orderOptions.forEach(function (option, index) {
                url += (option.direction === OrderDirection.DESC ? "-" : "") + option.field;
                if (index < orderOptionsLength - 1) {
                    url += ",";
                }
            });
        }
        if (filtersLength > 0) {
            if (started) {
                url += '&';
            }
            else {
                started = true;
            }
            var keys_1 = Object.keys(options.filter);
            keys_1.forEach(function (key, index) {
                var val = options.filter[key];
                url += "filter[" + key + "]=" + val;
                if (index < keys_1.length - 1) {
                    url += "&";
                }
            });
        }
        if (options.page) {
            if (started) {
                url += '&';
            }
            else {
                started = true;
            }
            url += 'page=' + options.page;
        }
        if (options.range) {
            if (started) {
                url += '&';
            }
            else {
                started = true;
            }
            url += 'range=' + options.range;
        }
        if (options.offset) {
            if (started) {
                url += '&';
            }
            url += 'offset=' + options.offset;
        }
        request.open("GET", url, true);
        this.addHeaders(request);
        var subject = new ReplaySubject(1);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    subject.next(_this.extractCollection(request.responseText, paginator));
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler, {
                        response: JSON.parse(request.responseText)
                    });
                }
            }
        };
        request.send();
        return subject;
    };
    /**
     * Load a collection in http service
     * @param {string} type Endpoint name
     * @param {{[p: string]: any}} filter Filter Object
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<CollectionDataSet>} Observable returning the collection data
     */
    Http.prototype.loadCollection = function (type, filter, errorHandler) {
        var _this = this;
        if (filter === void 0) { filter = {}; }
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.apiUrl(type) + type;
        var filterKeys = Object.keys(filter);
        if (filterKeys.length > 0) {
            url += "?";
        }
        filterKeys.forEach(function (key, index) {
            var val = filter[key];
            url += "filter[" + key + "]=" + val;
            if (index < filterKeys.length - 1) {
                url += "&";
            }
        });
        request.open("GET", url, true);
        this.addHeaders(request);
        var subject = new ReplaySubject(1);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    subject.next(_this.extractCollection(request.responseText));
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler, {
                        response: JSON.parse(request.responseText)
                    });
                }
            }
        };
        request.send();
        return subject;
    };
    /**
     * Save entity to the http service
     * @param {EntityDataSet} entity Entity data to save
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the entity data
     */
    Http.prototype.saveEntity = function (entity, type, id, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.apiUrl(type) + type + "/" + id;
        request.open("PATCH", url, true);
        this.addHeaders(request);
        var subject = new ReplaySubject(1);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    subject.next(_this.extractEntity(request.responseText));
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler, {
                        response: JSON.parse(request.responseText)
                    });
                }
            }
        };
        request.send(JSON.stringify(entity));
        return subject;
    };
    /**
     * Create entity in http service
     * @param {string} type Endpoint name
     * @param {EntityDataSet} data Data used to create the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the entity data
     */
    Http.prototype.createEntity = function (type, data, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.apiUrl(type) + type;
        request.open("POST", url, true);
        this.addHeaders(request);
        var subject = new ReplaySubject(1);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    subject.next(_this.extractEntity(request.responseText));
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler, {
                        response: JSON.parse(request.responseText)
                    });
                }
            }
        };
        request.send(JSON.stringify(data));
        return subject;
    };
    /**
     * Delete entity from http service
     * @param {string} type Endpoint type
     * @param {number} id Entity id
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<boolean>} True if deletion success
     */
    Http.prototype.deleteEntity = function (type, id, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var request = new XMLHttpRequest();
        var url = "" + this.apiUrl(type) + type + "/" + id;
        request.open("DELETE", url, true);
        this.addHeaders(request);
        var subject = new ReplaySubject(1);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    subject.next(true);
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler, {
                        response: JSON.parse(request.responseText)
                    });
                }
            }
        };
        request.send();
        return subject;
    };
    /**
     * Authenticate in service
     * @param {string} login User login
     * @param {string} password User password
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<boolean>} True if authentication success
     */
    Http.prototype.authenticate = function (login, password, errorHandler) {
        var _this = this;
        if (errorHandler === void 0) { errorHandler = null; }
        var subject = new ReplaySubject(1);
        var request = new XMLHttpRequest();
        var url = this.configuration.apiUrl + "api/login-token";
        request.open("GET", url, true);
        request.setRequestHeader("Authorization", 'Basic ' + btoa(login.trim() + ':' + password));
        var observables = [];
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    var loginData = JSON.parse(request.responseText);
                    var expire = +loginData["expires_in"] - 3600;
                    if (expire < 3600) {
                        if (localStorage.getItem(_this.interfaceName + "_accessToken")) {
                            observables.push(_this.setToken(loginData["access_token"], errorHandler));
                            _this.setExpireDate(expire);
                            _this.setRefreshToken(loginData["refresh_token"]);
                        }
                        observables.push(_this.refreshToken(loginData["refresh_token"], errorHandler));
                    }
                    else {
                        observables.push(_this.setToken(loginData["access_token"], errorHandler));
                        _this.setExpireDate(expire);
                        _this.setRefreshToken(loginData["refresh_token"]);
                    }
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler, {
                        response: JSON.parse(request.responseText)
                    });
                }
                combineLatest.apply(void 0, observables).map(function (values) {
                    return values[0];
                }).subscribe(function (data) {
                    subject.next(data);
                });
            }
        };
        request.send();
        return subject;
    };
    Http.prototype.logout = function () {
        // TODO: revoir cette partie du logout
        /*let keys:string[] = Object.keys(this.headers);

        keys.forEach((headerName:string) => {
            if (headerName !== "access-token") {
                delete this.headers[headerName];
            }
        });*/
        localStorage.removeItem(this.interfaceName + "_currentUser");
        localStorage.removeItem(this.interfaceName + "_accessToken");
        localStorage.removeItem(this.interfaceName + "_expires_in");
        localStorage.removeItem(this.interfaceName + "_refreshToken");
        this.dataStore.user = null;
        return new BehaviorSubject(true);
    };
    /**
     *
     * @param {string} accessToken
     * @param {Function} errorHandler
     */
    Http.prototype.setToken = function (accessToken, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        if (accessToken && accessToken != "") {
            localStorage.setItem(this.interfaceName + "_accessToken", JSON.stringify(accessToken));
            this.headers["access-token"] = accessToken;
            return this.getMe(true, errorHandler);
        }
    };
    /**
     *
     * @param {number} expire
     */
    Http.prototype.setExpireDate = function (expire) {
        var date = Date.now();
        localStorage.setItem(this.interfaceName + "_expires_in", JSON.stringify(date + (expire * 1000)));
    };
    /**
     *
     * @param {string} refreshToken
     * @param {Function} errorHandler
     */
    Http.prototype.refreshToken = function (refreshToken, errorHandler) {
        var _this = this;
        var subject = new ReplaySubject(1);
        var request = new XMLHttpRequest();
        var url = this.configuration.apiUrl + "api/refresh-token/" + refreshToken;
        request.open("GET", url, true);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    var userData = JSON.parse(request.responseText);
                    _this.setToken(userData["access_token"], errorHandler);
                    _this.setExpireDate(+userData["expires_in"] - 3600);
                    _this.setRefreshToken(userData["refresh_token"]);
                    subject.next(userData);
                }
                else {
                    _this.sendError(request.status, request.statusText, errorHandler, {
                        response: JSON.parse(request.responseText)
                    });
                }
            }
        };
        request.send();
        return subject;
    };
    /**
     *
     * @param {string} refreshToken
     */
    Http.prototype.setRefreshToken = function (refreshToken) {
        localStorage.setItem(this.interfaceName + "_refreshToken", JSON.stringify(refreshToken));
    };
    /**
     *
     * @param {boolean} complete
     * @param {Function} errorHandler
     * @returns {Observable<EntityDataSet>}
     */
    Http.prototype.getMe = function (complete, errorHandler) {
        var _this = this;
        if (complete === void 0) { complete = true; }
        if (errorHandler === void 0) { errorHandler = null; }
        var subject = new ReplaySubject(1);
        var request = new XMLHttpRequest();
        var url = this.configuration.apiUrl + "api/users/me";
        request.open("GET", url, true);
        this.addHeaders(request);
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE) {
                if (request.status === 200) {
                    var userData = JSON.parse(request.responseText)["data"][0];
                    subject.next(userData);
                    _this.setMe(userData, complete);
                }
                else {
                    if (errorHandler) {
                        _this.sendError(request.status, request.statusText, errorHandler, {
                            response: JSON.parse(request.responseText)
                        });
                    }
                    subject.error(null);
                }
            }
        };
        request.send();
        return subject;
    };
    /**
     *
     * @param {EntityDataSet} userData
     * @param {boolean} complete
     */
    Http.prototype.setMe = function (userData, complete) {
        if (complete === void 0) { complete = true; }
        if (complete) {
            this.dataStore.user = userData;
            //this.data.next(this.dataStore.user);
            localStorage.setItem(this.interfaceName + "_currentUser", JSON.stringify(userData));
        }
        //this.currentUserData = userData;
    };
    /**
     * Extract entity data from raw data
     * @param {string} responseText Response text from server
     * @returns {EntityDataSet} Entity data
     */
    Http.prototype.extractEntity = function (responseText) {
        var data = JSON.parse(responseText);
        // pas sûr que ce code serve
        if (data["data"][0] && data["data"][0]["id"] !== undefined) {
            data["data"][0]["id"] = data["data"][0]["id"];
        }
        if (data["data"] && data["data"]["id"] !== undefined) {
            data["data"]["id"] = data["data"]["id"];
        }
        if (data["data"][0]) {
            return data["data"][0];
        }
        else {
            return data["data"];
        }
    };
    /**
     * Extract collection data from raw data
     * @param {string} responseText Response text from server
     * @returns {CollectionDataSet} Collection data
     */
    Http.prototype.extractCollection = function (responseText, paginator) {
        if (paginator === void 0) { paginator = null; }
        var data = JSON.parse(responseText);
        if (paginator) {
            paginator.updateCount(+data["count"]);
        }
        var collectionData = {};
        data["data"].forEach(function (entityData) {
            // ?????
            entityData["id"] = entityData["id"];
            collectionData["_" + entityData["id"]] = entityData;
        });
        return collectionData;
    };
    /**
     * Callback appelé lors du déclanchement d'un event de type StorageEvent. Cela n'a lieu que si le localstorage a été modifié depuis une autre page.
     * Dans ce cas, nous controllons s'il s'agit d'une déconnexion pour notifier ensuite les abonnées a OnUnexpectedLogout
     * @param event
     * @return
     */
    Http.prototype.OnUnexpectedStorageChange = function (event) {
        if (event.key === this.interfaceName + "_accessToken"
            && event.newValue === null
            && event.oldValue !== null) {
            this.unexpectedLogoutSubject.next();
        }
    };
    return Http;
}(ExternalInterface));
export { Http };
//# sourceMappingURL=http.class.js.map