import { ExternalInterface } from "../abstract-external-interface.class";
import { DataConnector } from "../../data-connector.class";
import { NodejsConfiguration } from "./nodejs-configuration.interface";
import { Observable } from "rxjs/Rx";
import { CollectionDataSet, EntityDataSet } from "../../types";
/**
 * Nodejs interface
 */
export declare class Nodejs extends ExternalInterface {
    private configuration;
    private connector;
    private messagePrefix;
    private retrieveEvent;
    private connectionCommand;
    private socket;
    private errorsStore;
    private collectionsErrorStore;
    private connected;
    private collectionSubjects;
    private dataCollections;
    private collectionFilters;
    /**
     * Create the nodejs interface
     * @param {NodejsConfiguration} configuration Interface configuration object
     * @param {DataConnector} connector Reference to the connector
     */
    constructor(configuration: NodejsConfiguration, connector: DataConnector);
    clear(): void;
    private initializeSocket(forced?);
    /**
     * Send interface errors
     */
    private dispatchErrors();
    /**
     * Load entity in nodejs service
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the data
     */
    loadEntity(type: string, id: number, errorHandler: Function): Observable<EntityDataSet>;
    saveEntity(data: EntityDataSet, type: string, id: number, errorHandler?: Function): Observable<EntityDataSet>;
    /**
     * Load a collection in nodejs service
     * @param {string} type Endpoint name
     * @param {{[p: string]: any}} filter Filter Object
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<CollectionDataSet>} Observable returning the collection data
     */
    loadCollection(type: string, filter?: {
        [key: string]: any;
    }, errorHandler?: Function): Observable<CollectionDataSet>;
    /**
     * Create entity in nodejs service
     * @param {string} type Endpoint name
     * @param {EntityDataSet} data Data used to create the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the entity data
     */
    createEntity(type: string, data: EntityDataSet, errorHandler?: Function): Observable<EntityDataSet>;
    /**
     * Delete entity from nodejs service
     * @param {string} type Endpoint type
     * @param {number} id Entity id
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<boolean>} True if deletion success
     */
    deleteEntity(type: string, id: number, errorHandler?: Function): Observable<boolean>;
}
