var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ExternalInterface } from "../abstract-external-interface.class";
import * as ObjectHash from "object-hash";
import * as io from 'socket.io-client';
import { ReplaySubject } from "rxjs/Rx";
/**
 * Nodejs interface
 */
var Nodejs = /** @class */ (function (_super) {
    __extends(Nodejs, _super);
    //private temporaryCollectionsStore:{[key:string]:ReplaySubject<CollectionDataSet>} = {};
    /**
     * Create the nodejs interface
     * @param {NodejsConfiguration} configuration Interface configuration object
     * @param {DataConnector} connector Reference to the connector
     */
    function Nodejs(configuration, connector) {
        var _this = _super.call(this) || this;
        _this.configuration = configuration;
        _this.connector = connector;
        _this.errorsStore = {};
        _this.collectionsErrorStore = {};
        _this.connected = true;
        _this.collectionSubjects = {};
        _this.dataCollections = {};
        _this.collectionFilters = {};
        _this.messagePrefix = _this.configuration.messagePrefix || "message";
        _this.retrieveEvent = _this.configuration.retrievePrefix || "retrieve_";
        _this.connectionCommand = _this.configuration.connectionCommand || "connexion";
        _this.maxRetry = 1000;
        _this.initializeSocket();
        return _this;
    }
    Nodejs.prototype.clear = function () {
        this.errorsStore = {};
        this.collectionsErrorStore = {};
        this.collectionSubjects = {};
        this.dataCollections = {};
        this.collectionFilters = {};
    };
    Nodejs.prototype.initializeSocket = function (forced) {
        var _this = this;
        if (forced === void 0) { forced = false; }
        if (this.socket && !forced) {
            return;
        }
        if (this.socket) {
            this.socket.disconnect();
            this.socket.close();
        }
        this.socket = io(this.configuration.socketUrl);
        this.socket.on('connect_failed', function () {
            console.log('Connection Failed');
        });
        this.socket.on('connect_error', function (e) {
            console.log('Connection Error');
        });
        this.socket.on('connect', function () {
            console.log('Connected');
            _this.connected = true;
        });
        this.socket.on('reconnecting', function () {
            console.log('reconnecting');
        });
        this.socket.on('disconnect', function () {
            console.log('Disconnected');
            _this.connected = false;
            _this.dispatchErrors();
        });
        this.socket.on(this.messagePrefix, function (data) {
            if (data["id"] && data["data"]) {
                data["data"]["id"] = data["id"];
            }
            if (data['command'] === "put") {
                _this.connector.registerEntityByData(data["type"], data["id"] || data["data"]["id"], data['data']);
                if (data["type"] === _this.connector.configuration.liveRefreshService) {
                    _this.connector.refreshCollectionWithData(data["data"]["myType"], data["data"]);
                }
            }
            if (data['command'] === "update") {
                _this.connector.registerEntityByData(data["type"], data["id"] || data["data"]["id"], data['data']);
            }
            if (data['command'] === "delete") {
                _this.connector.unregisterEntityTypeAndId(data["type"], data["id"]);
            }
        });
    };
    /**
     * Send interface errors
     */
    Nodejs.prototype.dispatchErrors = function () {
        // error managing
        for (var id in this.errorsStore) {
            if (this.errorsStore.hasOwnProperty(id)) {
                this.sendError(0, "", this.errorsStore[id]);
                delete this.errorsStore[id];
            }
        }
        for (var id in this.collectionsErrorStore) {
            if (this.collectionsErrorStore.hasOwnProperty(id)) {
                this.sendError(0, "", this.collectionsErrorStore[id]);
                //delete this.collectionsErrorStore[id];
            }
        }
    };
    /**
     * Load entity in nodejs service
     * @param {string} type Endpoint name
     * @param {number} id Id of the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the data
     */
    Nodejs.prototype.loadEntity = function (type, id, errorHandler) {
        if (!this.connected) {
            this.sendError(0, '', errorHandler);
        }
        return this.loadCollection(type, {
            id: id
        }).map(function (data) {
            return data[0];
        });
    };
    Nodejs.prototype.saveEntity = function (data, type, id, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        var subject = new ReplaySubject(1);
        var cid = Math.floor(Math.random() * Number.MAX_SAFE_INTEGER);
        if (!this.connected) {
            this.sendError(0, '', errorHandler);
        }
        else {
            this.errorsStore[cid] = errorHandler;
            var requestData = {
                command: "update",
                data: data,
                id: id,
                type: type,
                cid: cid
            };
            this.socket.emit("message", requestData);
        }
        return subject;
    };
    /**
     * Load a collection in nodejs service
     * @param {string} type Endpoint name
     * @param {{[p: string]: any}} filter Filter Object
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<CollectionDataSet>} Observable returning the collection data
     */
    Nodejs.prototype.loadCollection = function (type, filter, errorHandler) {
        // à voir si il y a besoin d'initialiser
        //this.initializeSocket();
        var _this = this;
        if (filter === void 0) { filter = {}; }
        if (errorHandler === void 0) { errorHandler = null; }
        var hash = ObjectHash(filter);
        var subject = new ReplaySubject(1);
        if (!this.dataCollections[type]) {
            this.dataCollections[type] = {};
        }
        if (!this.collectionFilters[type]) {
            this.collectionFilters[type] = {};
        }
        if (!this.collectionSubjects[type]) {
            this.collectionSubjects[type] = {};
        }
        this.collectionSubjects[type][hash] = subject;
        this.collectionFilters[type][hash] = filter;
        if (!this.connected) {
            this.sendError(0, '', errorHandler);
        }
        else {
            var cid = Math.floor(Math.random() * Number.MAX_SAFE_INTEGER);
            var evtName_1 = this.retrieveEvent + type;
            this.socket.emit('connexion', type, filter, cid);
            var callback_1 = function (data) {
                var res = {};
                for (var id in data) {
                    var mid = data[id]["data"]["id"] || data[id]["id"];
                    res[mid] = data[id]["data"];
                }
                _this.dataCollections[type][hash] = res;
                subject.next(res);
                _this.socket.off(evtName_1, callback_1);
            };
            this.socket.off(evtName_1, callback_1);
            this.collectionsErrorStore[cid] = errorHandler;
            this.socket.on(evtName_1, callback_1);
        }
        return subject;
    };
    /**
     * Create entity in nodejs service
     * @param {string} type Endpoint name
     * @param {EntityDataSet} data Data used to create the entity
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<EntityDataSet>} Observable returning the entity data
     */
    Nodejs.prototype.createEntity = function (type, data, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        var subject = new ReplaySubject(1);
        var cid = Math.floor(Math.random() * Number.MAX_SAFE_INTEGER);
        var id = Math.floor(Math.random() * Number.MAX_SAFE_INTEGER);
        this.errorsStore[cid] = errorHandler;
        if (!this.connected) {
            this.sendError(0, '', errorHandler);
        }
        else {
            var requestData = {
                command: "put",
                data: data,
                type: type,
                cid: cid,
                id: id
            };
            this.socket.emit("message", requestData);
        }
        return subject;
    };
    /**
     * Delete entity from nodejs service
     * @param {string} type Endpoint type
     * @param {number} id Entity id
     * @param {Function} errorHandler Function used to handle errors
     * @returns {Observable<boolean>} True if deletion success
     */
    Nodejs.prototype.deleteEntity = function (type, id, errorHandler) {
        if (errorHandler === void 0) { errorHandler = null; }
        var subject = new ReplaySubject(1);
        var cid = Math.floor(Math.random() * Number.MAX_SAFE_INTEGER);
        this.errorsStore[cid] = errorHandler;
        if (!this.connected) {
            this.sendError(0, '', errorHandler);
        }
        else {
            var requestData = {
                command: "delete",
                id: id,
                type: type,
                cid: cid
            };
            this.socket.emit("message", requestData);
        }
        return subject;
    };
    return Nodejs;
}(ExternalInterface));
export { Nodejs };
//# sourceMappingURL=nodejs.class.js.map