/**
 * Created by Christophe on 10/10/2017.
 */
import { DataConnector } from "../data-connector.class";
import { DataEntity } from "./data-entity.class";
import { CollectionDataSet, EntityDataSet } from "../types";
import { Observable } from "rxjs/Observable";
import { ModelSchema } from "octopus-model";
/**
 * Data collection object
 */
export declare class DataCollection {
    type: string;
    private connector;
    paginated: boolean;
    count: number;
    /**
     * Entities contained by the collection
     * @type {any[]}
     */
    entities: DataEntity[];
    /**
     * Observables of entities contained by the collection
     * @type {any[]}
     */
    entitiesObservables: Observable<DataEntity>[];
    /**
     * Creates the collection
     * @param {string} type
     * @param {CollectionDataSet | EntityDataSet[]} data
     * @param {DataConnector} connector
     * @param {ModelSchema} structure
     */
    constructor(type: string, data: CollectionDataSet | EntityDataSet[], connector?: DataConnector, structure?: ModelSchema, embeddings?: {
        [key: string]: string;
    });
    /**
     * Remove entity from collection
     * @param {number} id Id of the entity to delete
     */
    deleteEntity(id: number | string): void;
    /**
     * Register entity in collection, if not already contained by the collection
     * @param {DataEntity} entity Entity to register
     * @param {Observable<DataEntity>} entityObservable Entity observable to register
     */
    registerEntity(entity: DataEntity, entityObservable: Observable<DataEntity>): void;
}
