import { DataEntity } from "./data-entity.class";
/**
 * Data collection object
 */
var DataCollection = /** @class */ (function () {
    /**
     * Creates the collection
     * @param {string} type
     * @param {CollectionDataSet | EntityDataSet[]} data
     * @param {DataConnector} connector
     * @param {ModelSchema} structure
     */
    function DataCollection(type, data, connector, structure, embeddings) {
        if (connector === void 0) { connector = null; }
        if (structure === void 0) { structure = null; }
        if (embeddings === void 0) { embeddings = null; }
        var _this = this;
        this.type = type;
        this.connector = connector;
        this.paginated = false;
        /**
         * Entities contained by the collection
         * @type {any[]}
         */
        this.entities = [];
        /**
         * Observables of entities contained by the collection
         * @type {any[]}
         */
        this.entitiesObservables = [];
        if (Array.isArray(data)) {
            data.forEach(function (elem) {
                if (structure) {
                    elem = structure.filterModel(elem);
                }
                _this.entities.push(new DataEntity(type, elem, connector, elem["id"], embeddings));
            });
        }
        else {
            var keys = Object.keys(data);
            keys.forEach(function (key) {
                if (structure) {
                    data[key] = structure.filterModel(data[key]);
                }
                _this.entities.push(new DataEntity(type, data[key], connector, key, embeddings));
            });
        }
    }
    /**
     * Remove entity from collection
     * @param {number} id Id of the entity to delete
     */
    DataCollection.prototype.deleteEntity = function (id) {
        var _this = this;
        this.entities.forEach(function (entity, index) {
            if (entity.id === id) {
                _this.entities.splice(index, 1);
                _this.entitiesObservables.splice(index, 1);
            }
        });
    };
    /**
     * Register entity in collection, if not already contained by the collection
     * @param {DataEntity} entity Entity to register
     * @param {Observable<DataEntity>} entityObservable Entity observable to register
     */
    DataCollection.prototype.registerEntity = function (entity, entityObservable) {
        var count = 0;
        for (var _i = 0, _a = this.entities; _i < _a.length; _i++) {
            var collectionEntity = _a[_i];
            if (entity.id && entity.id === collectionEntity.id) {
                this.entities[count] = entity;
                this.entitiesObservables[count] = entityObservable;
                return;
            }
            count++;
        }
        this.entities.push(entity);
        this.entitiesObservables.push(entityObservable);
    };
    return DataCollection;
}());
export { DataCollection };
//# sourceMappingURL=data-collection.class.js.map