/**
 * Created by Christophe on 10/10/2017.
 */
import { DataConnector } from "../data-connector.class";
import { Observable } from "rxjs/Rx";
import { EntityDataSet } from "../types";
/**
 * Data entity unit object
 */
export declare class DataEntity {
    type: string;
    private connector;
    id: number | string;
    private embeddingsConf;
    /**
     * Entity attributes
     * @type {{}}
     */
    attributes: {
        [key: string]: any;
    };
    /**
     * Nested entities
     * @type {{}}
     */
    nesting: {
        [key: string]: any;
    };
    relationship: {
        [key: string]: DataEntity;
    };
    /**
     * Reference object for diff
     */
    private attributesRef;
    /**
     *
     * @type {{}}
     */
    private embeddings;
    /**
     * Create the data entity
     * @param {string} type Type of the entity
     * @param {EntityDataSet} data Entity data
     * @param {DataConnector} connector Reference to the connector
     * @param {number} id Entity id
     * @param {Object} embeddingsConf
     */
    constructor(type: string, data: EntityDataSet, connector?: DataConnector, id?: number | string, embeddingsConf?: {
        [key: string]: string;
    });
    /**
     * Set an attribute by key
     * @param {string} key Key name
     * @param value New value
     */
    set(key: string, value: any): void;
    /**
     * Get an attribute by key
     * @param {string} key Key name
     * @returns {any} Value
     */
    get(key: string): any;
    /**
     *
     * @returns {boolean}
     */
    readonly hasChanges: boolean;
    /**
     *
     * @param {string} name
     * @returns {DataEntity | DataEntity[]}
     */
    getEmbed(name: string): DataEntity | DataEntity[];
    /**
     * Save the entity
     * @returns {Observable<DataEntity>} The observable associated to the entity in connector stores
     */
    save(forceReload?: boolean, dispatchBeforeResponse?: boolean): Observable<DataEntity>;
    saveAction(forceReload?: boolean, dispatchBeforeResponse?: boolean): Observable<DataEntity>;
    /**
     * Delete the entity
     * @returns {Observable<boolean>} True if deletion success
     */
    remove(): Observable<boolean>;
    /**
     * Copy the attributes to generate the new reference object (for diff)
     */
    private generateReferenceObject();
    /**
     * Get an attributes cloned object
     * @returns {EntityDataSet} The cloned attributes object
     */
    getClone(): EntityDataSet;
    /**
     * Return the diff (only updated properties since last save action)
     * @returns {EntityDataSet} Diff object
     */
    getDiff(): EntityDataSet;
}
