import { DataEntity } from "../data-structures/data-entity.class";
import { Observable } from "rxjs/Observable";
import { ReplaySubject } from "rxjs/Rx";
/**
 * Entity store: where the entities and the entities observables are stored for an endpoint
 */
export declare class EntityStore {
    /**
     * Stores entities subjects, indexed by entity id
     * @type {{}}
     */
    private entitiesObservables;
    /**
     * Creates the store
     */
    constructor();
    clear(): void;
    /**
     * Registers the entity in store and update associated subject. If the subject does not exists, creates it
     * @param {DataEntity} entity Entity to register
     * @param {number} id Entity id
     * @returns {Observable<DataEntity>} Observable associated to the entity
     */
    registerEntity(entity: DataEntity, id: number | string): Observable<DataEntity>;
    /**
     * Register an entity subject to a specified id
     * @param {number} id Id used for registration
     * @param {ReplaySubject<DataEntity>} subject Subject to register
     */
    registerEntitySubject(id: number, subject: ReplaySubject<DataEntity>): void;
    /**
     * Delete all data for a specific entity id
     * @param {number} id Entity id
     */
    unregister(id: number | string): void;
    /**
     * Delete entity from store
     * @param {DataEntity} entity Entity to delete
     */
    unregisterEntity(entity: DataEntity): void;
    /**
     * Returns the observable associated to the specified id
     * @param {number} id Id used in registration
     * @returns {Observable<DataEntity>} Entity subject associated to the id
     */
    getEntityObservable(id: number | string, createObservable?: boolean): ReplaySubject<DataEntity>;
    /**
     * Returns true if entity is defined in store
     * @param {number} entityId Entity id
     * @returns {boolean}
     */
    isInStore(entityId: number): boolean;
}
